#!/usr/bin/env python
# -*- coding: utf-8 -*-

__version__ = '0.1'
import os,sys,re,time
from mechanize import Browser
from urlparse import urljoin,urlparse
import optparse


seen=[]
stats=[]
b=Browser()

def is_local(url,url2):
    return urlparse(url).netloc == urlparse(url2).netloc

def srcs(doc):
    return [m[1] for m in re.findall(r'\ssrc\s*=\s*(["\'])(.*?)\1',doc)]

def linkcheck(url,checks,media,rec_limit):
    start = time.time()
    seen.append(url)
    try:
        resp = b.open(url)
        stats.append((time.time()-start,url))
        print url,'ok','%.2fs' % (time.time()-start)
        if rec_limit:
            for lnk in list(b.links()):
                lnk = urljoin(url,lnk.url)
                if lnk not in seen and all(f(lnk) for f in checks):
                    linkcheck(lnk,checks,is_local(lnk,url) and media or False,is_local(lnk,url) and rec_limit-1 or 0)
        if media:
            for lnk in srcs(resp.read()):
                lnk = urljoin(url,lnk)
                linkcheck(lnk,checks,False,0)
    except Exception,e:
        if str(e) != 'not viewing HTML':
            print >>sys.stderr,url,e
        else:
            stats.append((time.time()-start,url))


def main():
    global opts
    op=optparse.OptionParser(usage="%prog url",version=__version__)
    op.add_option('--match',default='',
            help="check only links matching regexp")
    op.add_option('--ignore',default='',
            help="ignore links matching regexp")
    op.add_option('-l','--local-only',action='store_true',default=False,
            help="Check pages from same domain only")
    op.add_option('-v','--verbose',action='store_true',default=False,
            help="Print successful request too")
    op.add_option('-r','--recursion-limit',type='int',default=999)
    op.add_option('--ignore-robots',action='store_true',default=False,
            help="ignore robots.txt")
    op.add_option('-m','--media',action='store_true',default=False,
            help="Check media files(images,stylesheets,javascripts,etc.)")

    opts,args = op.parse_args(sys.argv[1:])
    if not(args) or len(args)>1:
        op.print_help()
        sys.exit(1)
    home=args[0]

    checks=[]
    if not opts.verbose:
        sys.stdout = open(os.devnull,'w')
    b.set_handle_robots(opts.ignore_robots)
    if opts.match:
        checks.append(re.compile(opts.match).match)
    if opts.ignore:
        checks.append(lambda url: not re.match(opts.ignore,url))
    if opts.local_only:
        checks.append(lambda url: is_local(url,home))

    linkcheck(home,checks,opts.media,opts.recursion_limit)
    if stats:
        stats.sort()
        print 'Average response time: %.2fs' % (sum(x[0] for x in stats)/len(stats))
        print 'Shortest response time: %.2fs (%s)' % stats[0]
        print 'Longest response time: %.2fs (%s)' % stats[-1]

if __name__ == '__main__':
    main()



